/*
 * juked - a jukebox daemon
 * Copyright (C) 2000-2002 Sam Clegg - See COPYING
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 * $superduper: database.c,v 1.26 2002/05/01 11:37:52 samc Exp $
 */
#include "config.h"

#include "juked.h"
#include "database.h"
#include "db/db_mysql.h"
#include "db/db_builtin.h"

/**
 * Set up the database function pionters according the 
 * database type being used.
 * @return 0 on success, -1 on failure.
 */
int 
init_database ()
{
  if (!settings.db_type) return -1;
  if (!strncmp (settings.db_type, "mysql", 5))
    {
#ifdef HAVE_MYSQL
      term_database    = term_database_mysql;
      db_is_valid_id   = db_is_valid_id_mysql;
      db_get_filename  = db_get_filename_mysql;
      db_get_random_id = db_get_random_id_mysql;
      db_search        = db_search_mysql;
      db_lookup        = db_lookup_mysql;
      db_list          = db_list_mysql;
      return init_database_mysql ();
#else
      jdmessage (LOG_ERR, "this version of juked was not compiled with MySQL support\n");
      return -1;
#endif
    }
  else if (!strncmp (settings.db_type, "built-in", 7))
    {
      term_database    = term_database_builtin;
      db_is_valid_id   = db_is_valid_id_builtin;
      db_get_filename  = db_get_filename_builtin;
      db_get_random_id = db_get_random_id_builtin;
      db_search        = db_search_builtin;
      db_lookup        = db_lookup_builtin;
      db_list          = db_list_builtin;
      return init_database_builtin ();
    }
  else 
    {
      jdmessage (LOG_ERR, "unknown database type: %s\n", settings.db_type);
      return -1;
    }
}
