/*
 * juked - a jukebox daemon
 * Copyright (C) 2000-2002 Sam Clegg - See COPYING
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 * Main header file which is included from all juked source files.
 * Only header and #defines that are used throughout should be in this file.
 *
 * $superduper: juked.h,v 1.18 2002/05/01 11:37:02 samc Exp $
 */
#ifndef JUKED_H
#define JUKED_H

/* system headers */
#include <unistd.h>
#include <stdarg.h>
#include <stdlib.h>
#include <malloc.h>
#include <assert.h>
#include <errno.h>
#include <limits.h>
#include <string.h>
#include <ctype.h>
#include <syslog.h>
#include <pthread.h>

/* libxml2 headers */
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xmlmemory.h>

/* global juked headers */
#include "settings.h"

/* we stick to the GNU standard */
#define _GNU_SOURCE
/* prefix of ouput for unscuccessful command */
#define ERROR_STR  "-ERR "
/* prefix of ouput for successful command */
#define OK_STR     "+OK "
/* max number of symultanious connections [FIXME] make this dynamic */
#define NUM_CONNECTIONS 10 

/* xml constants */
#define XML_BASENODE_CONF  "jukedconf"
#define XML_BASENODE_LIST  "jukedlist"
#define XML_BASENODE_INFO  "jukedinfo"
#define XML_BASENODE_DATA  "jukeddata"
#define XML_DTD_URI        "http://www.superduper.net/juked/DTD/"
#define XML_TAG_ARTIST     "artist"
#define XML_TAG_ALBUM      "album"
#define XML_TAG_TITLE      "title"
#define XML_TAG_COMMENT    "comment"
#define XML_TAG_TRACKID    "trackid"
#define XML_TAG_TRACKNUM   "tracknum"

/* Functions defined in juked.c and used everywhere */
void  jdmessage (int debuglvl, char *string, ...);
void  jdoutput (char *fmt, ...);
char  jdgetc ();

extern time_t startup_time;
extern char localhost[_POSIX_PATH_MAX];
extern pthread_key_t session_key;

#endif /* !JUKED_H */
