/*
 * juked - a jukebox daemon
 * Copyright (C) 2000-2002 Sam Clegg - See COPYING
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 * Lexer (tokenizer) functions for jcp (jukebox control protocol)
 * I had to write my own lexer bacause flex in not at all
 * thread safe.
 *
 * $superduper: jcp_lexer.c,v 1.5 2002/04/29 17:29:04 samc Exp $
 */
#include "juked.h"
#include "jcp_parser.h"

/**
 * The maximun allowed input line lenght.
 */
#define MAX_LINE 256

/**
 * keyword type used in keywork listings.
 */
struct keyword_type
{
  char string[256];
  int  token;
};

/**
 * The list of accepable keyword in jcp.
 */
static struct keyword_type keywords[] =
{
    { "user"    , TOK_USER     },
    { "pass"    , TOK_PASS     },
    { "play"    , TOK_PLAY     },
    { "stop"    , TOK_STOP     },
    { "pause"   , TOK_PAUSE    },
    { "next"    , TOK_NEXT     },
    { "prev"    , TOK_PREV     },
    { "info"    , TOK_INFO     },
    { "quit"    , TOK_QUIT     },
    { "exit"    , TOK_EXIT     },
    { "status"  , TOK_STATUS   },
    { "ladd"    , TOK_LADD     },
    { "lrm"     , TOK_LRM      },
    { "find"    , TOK_FIND     },
    { "artists" , TOK_ARTISTS  },
    { "llist"   , TOK_LLIST    },
    { "album"   , TOK_FIELD    },
    { "atrist"  , TOK_FIELD    },
    { "track"   , TOK_FIELD    },
    { "random"  , TOK_RANDOM   },
    { "history" , TOK_HISTORY  },
    { "help"    , TOK_HELP     },
    { "?"       , TOK_HELP     },
    { "version" , TOK_VERSION  },
    { "autoskip", TOK_SKIP     }
};

/**
 * The number of legal keyword.
 */
static int num_keywords = sizeof (keywords) / sizeof (struct keyword_type);

/**
 * String matching function
 * @param str1 the first string to compare
 * @param str2 the second string to compare
 * @param len the number of characters for try and match
 */
static int 
match (char* str1, char* str2, int len) 
{
  if (strncasecmp (str1, str2, len) == 0)
      return 1;
  else
      return 0;
}

/**
 * Read a string from the user
 * Stops on white space
 * @param string the string to read into
 * @return the number of characters read, 0 on EOL, -1 of EOF.
 */
static int
read_string (char* string)
{
  char c;
  int  len = 0;
  static int new_line_next = 0;   /* [FIXME] HACK */
  if (new_line_next)
    {
      new_line_next = 0;
      return 0;
    }
  while (len < MAX_LINE)
    {
      c = jdgetc ();
      if (c == -1) return -1;
      if (isspace (c)) 
        {
          if (len == 0) 
            {
              if (c == '\n') return 0;
              continue;
            }
          if (c == '\n') new_line_next = 1;
          break;
        }
      string[len] = c;
      len++;
    }
  string[len] = '\0';
  return len;
}

/**
 * lexer function called from the parser
 * @param lvalp the token structure to fill
 * @return token type or NO_TOKEN if match failed
 */
int
yylex (YYSTYPE *lvalp)
{
  int  i;
  int  len;
  char buf[MAX_LINE];

  /* read next string */
  len = read_string (buf);
  if (len == -1) return 0;
  if (len ==  0) return TOK_EOL;

  for (i=0; i<num_keywords; i++)
    {
      if (len != strlen (keywords[i].string)) continue;
      if (match (buf, keywords[i].string, len)) 
        {
          if (keywords[i].token == TOK_FIELD)
              lvalp->string = strdup (buf);
          return keywords[i].token;
        }
    }

  if (isdigit (buf[0]))
    {
      for (i=0; i<len; i++)
        {
          if (!isdigit (buf[i])) 
            {
              lvalp->string = strdup (buf);
              return TOK_NO_TOKEN;
            }
        }
      lvalp->num = atoi (buf);
      return TOK_SONGID;
    }
  lvalp->string = strdup (buf);
  return TOK_STRING;
}
