/*
 * juked - a jukebox daemon
 * Copyright (C) 2000-2002 Sam Clegg - See COPYING
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Parser for jukebox control protocol (jcp)
 * Juked uses this to parse client commands.
 *
 * $superduper: jcp_parser.y,v 1.5 2002/05/02 14:30:50 samc Exp $
 */
%{
#include "juked.h"
#include "commands.h"
#include "playlist.h"

void yyerror(char* string);
int  yylex(void *lvalp);
%}

%union 
{
  int   num;
  char* string;
}

%token <num> 
TOK_USER TOK_PASS
TOK_PLAY TOK_STOP TOK_PAUSE TOK_PREV TOK_NEXT TOK_EXIT
TOK_LADD TOK_LRM TOK_SONGID TOK_QUIT TOK_LLIST TOK_RANDOM 
TOK_FIND TOK_ARTISTS TOK_STATUS TOK_HISTORY TOK_HELP TOK_SKIP
TOK_EOL TOK_VERSION TOK_INFO '"'
%token <string> TOK_STRING TOK_FIELD TOK_NO_TOKEN

%start session
%pure_parser
%%

session	: commands 
	;

commands: commands command TOK_EOL
        | commands error TOK_EOL  { yyerrok; }
   	| /* empty list */
        ;

command : TOK_PLAY TOK_SONGID     { cmd_play(1, $2);  }
        | TOK_USER TOK_STRING     { cmd_user($2);     }
        | TOK_PASS TOK_STRING     { cmd_pass($2);     }
        | TOK_INFO TOK_SONGID     { cmd_info($2);     }
	| TOK_STOP                { cmd_stop(1);      }
	| TOK_NEXT                { cmd_next(1);      }
	| TOK_PREV                { cmd_prev(1);      }
	| TOK_PAUSE               { cmd_pause(1);     }
 	| TOK_LADD                { cmd_add(1, -1);   }
 	| TOK_LADD TOK_SONGID     { cmd_add(1, $2);   }
        | TOK_LRM TOK_SONGID      { cmd_remove(1, $2);}
        | TOK_STATUS              { cmd_status(1);    }
        | TOK_FIND TOK_FIELD TOK_STRING { cmd_find($2,$3); 
                                       free($2); free($3);    }
        | TOK_FIND TOK_STRING     { cmd_find(0,$2); free($2); }
        | TOK_QUIT                { cmd_quit(1); YYACCEPT;  }
        | TOK_EXIT                { cmd_quit(1); YYACCEPT;  }
        | TOK_RANDOM              { cmd_random(1);  }
        | TOK_SKIP                { cmd_autoskip(1);}
        | TOK_LLIST               { cmd_list   ();  }
        | TOK_ARTISTS             { cmd_artists();  }
        | TOK_VERSION             { cmd_version();  }
        | TOK_HELP                { cmd_help   ();  }
        | TOK_HISTORY             { cmd_history();  }
        | TOK_NO_TOKEN            { yyerror($1);    }
        ;
        //| FIND STRING '"' STRING '"' { cmd_find($2,$4); free($2); free($4); }

%%

void 
yyerror (char* string)
{
  jdmessage (LOG_DEBUG,"%s\n", string);
  jdoutput ("%s%s\n", ERROR_STR, string);
  jdoutput (".\n");
}
